==> galileo-ide-metrics-dashboard-widget-0.0.1.tgz: package/src/browser/components/MetricWidgetContext.tsx <==
import * as React from "react";
import {
  createContext,
  useContext,
  useEffect,
  useState,
} from "@theia/core/shared/react";
import {
  IGetValueOptions,
  IMetricWidgetContext,
  IMetricWidgetProvider,
  IResponseDict,
} from "./MetricWidget.interface";
import { EResponseFormat } from "../constants";
const MetricWidgetContext = createContext({} as IMetricWidgetContext);

export function MetricWidgetProvider({
  children,
  responseFormat,
  metricUrl,
}: IMetricWidgetProvider) {
  const [globalResponse, setGlobalResponse] = useState();
  const [globalResponseFormat, setGlobalResponseFormat] =
    useState<EResponseFormat>();
  const [responseDict, setResponseDict] = useState<IResponseDict>({});

  useEffect(() => {
    fetchMetricUrl(metricUrl, responseFormat, true);
  }, []);

  const fetchMetricUrl = async (
    url?: string,
    responseFormat?: EResponseFormat,
    isGlobal?: boolean
  ) => {
    if (url) {
      try {
        const response = await fetch(url);
        let data: any;
        if (responseFormat === EResponseFormat.PROMETHEUS) {
          data = await response.text();
        } else if (responseFormat === EResponseFormat.JSON) {
          data = await response.json();
        }
        if (isGlobal) {
          setGlobalResponse(data);
          setGlobalResponseFormat(responseFormat);
        } else {
          setResponseDict((prevState) => ({
            ...prevState,
            [url]: { data },
          }));
        }
      } catch (err) {
        console.error(err);
      }
    }
  };

  const getValueByKey = (options: IGetValueOptions): any => {
    const { responseFormat } = options;
    let value: any;
    let responseFormatTemp;
    responseFormatTemp = responseFormat || globalResponseFormat;
    if (responseFormatTemp === EResponseFormat.PROMETHEUS) {
      value = getValueFromPrometheus(options);
    } else if (responseFormatTemp === EResponseFormat.JSON) {
      value = getValueFromJson(options);
    }

    return value;
  };

  const getValueFromPrometheus = (options: IGetValueOptions) => {
    const { key, metricUrl } = options;
    let source;
    if (metricUrl) {
      source = responseDict[metricUrl]?.data;
    } else {
      source = globalResponse;
    }
    const dataArr = source.split("\n");
    const line = dataArr.find((e: any) => e.startsWith(key));
    if (!line) {
      return "";
    }
    const lineArr = line.split(" ");
    return lineArr[lineArr.length - 1];
  };

  const getValueFromJson = (options: IGetValueOptions) => {
    const { key, metricUrl, keySeparator } = options;
    let source;
    if (metricUrl) {
      source = responseDict[metricUrl]?.data;
    } else {
      source = globalResponse;
    }
    let value = source;
    if (value) {
      if (keySeparator) {
        const keyArr = key.split(keySeparator);
        keyArr.forEach((el) => (value = value ? value[el] : ""));
      } else {
        value = source[key];
      }
    }

    return value;
  };

  const initialState: IMetricWidgetContext = {
    globalResponse,
    globalResponseFormat,
    responseDict,
    fetchMetricUrl,
    getValueByKey,
  };

  return (
    <MetricWidgetContext.Provider value={initialState}>
      {children}
    </MetricWidgetContext.Provider>
  );
}

export const useMetricWidgetContext = () => useContext(MetricWidgetContext);

==> galileo-ide-metrics-dashboard-widget-1.0.0.tgz: package/src/browser/components/MetricWidgetContext.tsx <==
import * as React from "react";
import {
  createContext,
  useContext,
  useEffect,
  useState,
} from "@theia/core/shared/react";
import {
  IGetValueOptions,
  IMetricWidgetContext,
  IMetricWidgetProvider,
  IResponseDict,
} from "./MetricWidget.interface";
import { ELocalStorageKey, EResponseFormat } from "../constants";
const MetricWidgetContext = createContext({} as IMetricWidgetContext);

export function MetricWidgetProvider({
  children,
  responseFormat,
  metricUrl,
}: IMetricWidgetProvider) {
  const [globalResponse, setGlobalResponse] = useState();
  const [globalResponseFormat, setGlobalResponseFormat] =
    useState<EResponseFormat>();
  const [responseDict, setResponseDict] = useState<IResponseDict>({});

  useEffect(() => {
    localStorage.removeItem(ELocalStorageKey.FETCHED_URLS);
    fetchMetricUrl(metricUrl, responseFormat, true);
  }, []);

  const fetchMetricUrl = async (
    url?: string,
    responseFormat?: EResponseFormat,
    isGlobal?: boolean
  ) => {
    if (url && !getIsUrlFetched(url)) {
      try {
        saveMetricUrlToStorage(url);
        const response = await fetch(url);
        let data: any;
        if (responseFormat === EResponseFormat.PROMETHEUS) {
          data = await response.text();
        } else if (responseFormat === EResponseFormat.JSON) {
          data = await response.json();
        }
        if (isGlobal) {
          setGlobalResponse(data);
          setGlobalResponseFormat(responseFormat);
        } else {
          setResponseDict((prevState) => ({
            ...prevState,
            [url]: { data },
          }));
        }
      } catch (err) {
        console.error(err);
      }
    }
  };

  const saveMetricUrlToStorage = (url: string) => {
    let fetchedUrls = localStorage.getItem(ELocalStorageKey.FETCHED_URLS);
    if (fetchedUrls) {
      const fetchedUrlsParsed = JSON.parse(fetchedUrls);
      fetchedUrlsParsed.push(url);
      fetchedUrls = JSON.stringify(fetchedUrlsParsed);
    } else {
      fetchedUrls = JSON.stringify([url]);
    }
    localStorage.setItem(ELocalStorageKey.FETCHED_URLS, fetchedUrls);
  };

  const getIsUrlFetched = (url: string) => {
    let fetchedUrls = localStorage.getItem(ELocalStorageKey.FETCHED_URLS);
    if (fetchedUrls) {
      const fetchedUrlsParsed = JSON.parse(fetchedUrls);
      return fetchedUrlsParsed.indexOf(url) >= 0;
    } else {
      return false;
    }
  };

  const getValueByKey = (options: IGetValueOptions): any => {
    const { responseFormat } = options;
    let value: any;
    let responseFormatTemp;
    responseFormatTemp = responseFormat || globalResponseFormat;
    if (responseFormatTemp === EResponseFormat.PROMETHEUS) {
      value = getValueFromPrometheus(options);
    } else if (responseFormatTemp === EResponseFormat.JSON) {
      value = getValueFromJson(options);
    }

    return value;
  };

  const getValueFromPrometheus = (options: IGetValueOptions) => {
    const { key, metricUrl } = options;
    let source;
    if (metricUrl) {
      source = responseDict[metricUrl]?.data;
    } else {
      source = globalResponse;
    }
    const dataArr = source.split("\n");
    const line = dataArr.find((e: any) => e.startsWith(key));
    if (!line) {
      return "";
    }
    const lineArr = line.split(" ");
    return lineArr[lineArr.length - 1];
  };

  const getValueFromJson = (options: IGetValueOptions) => {
    const { key, metricUrl, keySeparator } = options;
    let source;
    if (metricUrl) {
      source = responseDict[metricUrl]?.data;
    } else {
      source = globalResponse;
    }
    let value = source;
    if (value) {
      if (keySeparator) {
        const keyArr = key.split(keySeparator);
        keyArr.forEach((el) => (value = value ? value[el] : ""));
      } else {
        value = source[key];
      }
    }

    return value;
  };

  const initialState: IMetricWidgetContext = {
    globalResponse,
    globalResponseFormat,
    responseDict,
    fetchMetricUrl,
    getValueByKey,
  };

  return (
    <MetricWidgetContext.Provider value={initialState}>
      {children}
    </MetricWidgetContext.Provider>
  );
}

export const useMetricWidgetContext = () => useContext(MetricWidgetContext);
